;*******************************************************************************
;****							File : Ex5.asm
;****	Use the Table Instruction to read the table content from Program Memory
;****   Send these table content to RS-232 with A/D ASCII result
;****
;****   Set Timer1 interrupt time : 1 SEC with High priority ISR
;****   Receiver of USARt using the Low priority ISR
;*******************************************************************************

			list 	p = 18F452
			include <p18F452.inc>
			
;
			CBLOCK	0x00
			C_Hold_Delay			; variable used for the sample hold charge time
			Flag_Reg				; Flag Register
			Hex_Temp				; Hex to ASICC convert buffer
			ENDC	

			CBLOCK	0x080
			WREG_TEMP				;variable used for context saving 
			STATUS_TEMP				;variable used for context saving
			BSR_TEMP				;variable used for context saving
			ENDC
;
#define		TMR1_VAL	.32768		; Define Timer1 time period are 1 SEC
#define		TxD_Flag	Flag_Reg,0	; Define the TX data flag
#define		Pause_Flag	Flag_Reg,1	; Define PAUSE flag for the USART
#define		Send_C_Flag Flag_Reg,2	; Define flag for received "c" from terminal 
#define		Send_P_Flag Flag_Reg,3	; Define flag for received "p" from terminal 
;
PUT_Address	MACRO	TARGET_STR
			movlw	UPPER	TARGET_STR
			movwf	TBLPTRU
			movlw	HIGH	TARGET_STR
			movwf	TBLPTRH
			movlw	LOW		TARGET_STR
			movwf	TBLPTRL
			call	Put_String		; 	
			ENDM
;					       		
;********************************************************************
;****	RESET Vector @ 0x0000
;********************************************************************

			org		0x0000 			; Reset Vector 
 			bra		Init
;
			org		0x0008			; High Priority Interrupt Vector
			bra		Hi_ISRs
;
			org		0x0018			; Low Priority Interrupt Vector
			bra		Low_ISRs
;
;********************************************************************
;****	The Main Program start from Here !! 
;********************************************************************

			org		0x0020
Init:
			call	Init_IO
			call	Init_Timer1
			call	Init_AD
			call	Init_USART
;
			PUT_Address	String_0
			PUT_Address	String_1
			PUT_Address	String_2
			PUT_Address	String_3
			PUT_Address	String_4
;		
			bsf		RCON,IPEN		; Enable Interrupt priority
			bsf		INTCON,GIEH		; Enable High Priority Interrupt for Timer1
			bsf		INTCON,GIEL		; Enable Low Priority Interrupt for RxD of USART
			clrf	Flag_Reg		; Clear all bit in falg register 
;
	
;
Main:	
			btfss	Send_C_Flag		; if = 1, send " The A/D is running !" to RS-232
			bra		Check_1
			bcf		Send_C_Flag		; clear the flag for send data
			PUT_Address	String_7
;
Check_1		btfss	Send_P_Flag		; if = 1, send " The A/D is stopped conversion !" to RS-232
			bra		Check_2
			bcf		Send_P_Flag		; clear the flag for send data
			PUT_Address	String_6
;		 
Check_2		btfsc	Pause_Flag 		; If = 1, stop to both A/D convert and send USART
			bra		Main			
;
			btfss	TxD_Flag 		; Check time period is 1 Sec?
			bra 	Main
;
;-------- Start to do the A/D convert and send the A/D result to USART   ----------
;
			bcf		TxD_Flag 		; Clear 1 SEC time out flag
			PUT_Address	String_5	; Dispaly string " The A/D Results are : "
;
			call	AD_Convert		
 			movf	ADRESH,W		; Conver A/D <b9:b8> to ASCII and send to USART
 			andlw	b'00000011'
 			call	Hex_ASCII
 			call 	Tx_a_Byte
;
 			swapf	ADRESL,W		; Conver A/D <b7:b4> to ASCII and send to USART
 			andlw	h'0F'
 			call	Hex_ASCII
 			call	Tx_a_Byte
;
 			movf	ADRESL,W		; Conver A/D <b3:b0> to ASCII and send to terminal
 			andlw	h'0F'
 			call	Hex_ASCII
 			call	Tx_a_Byte
;
			movlw	h'0a'			; Send 0x0A & 0x0D to terminal
			call 	Tx_a_Byte
			movlw	h'0d'
			call 	Tx_a_Byte
;
			bra		Main
			
			
;
;******   Send a byte to USART   ******
Tx_a_Byte:
			movwf	TXREG					; Send a byte vis USART
			nop								; 
			btfss	PIR1,TXIF				; Does the data sent out ?
			bra		$-4						; No, Loop check the TXIF
			bcf		PIR1,TXIF				; Yes, completed to send the date, clear the TXIF
			return
;
;******* Convert low nibble to ASCII code   *******
Hex_ASCII:
 			andlw	h'0F'					; Make sure high nibble are "0000"
 			movwf	Hex_Temp
 			movlw	h'9'					; Compare with 9
 			cpfsgt  Hex_Temp
 			bra		Less_9
 			movf	Hex_Temp,W				; > 9, the number plus 0x37
 			addlw	h'37'
 			return
Less_9		movf	Hex_Temp,W				; < = 9, the number plus 0x30
 			addlw	h'30'
 			return
 			
;
;*****    Connvert the 10-bit A/D   *****
AD_Convert:
			call	C_Hold_Time				; Delay 50uS for the sample hold time
			bsf		ADCON0,GO				; Start convert the A/D
			nop								; Nop
			btfsc	ADCON0,GO				; Check the A/D completed ?
			bra		$-4						; A/D isn't completed yet, continut check.
			return 
;
;***********************************************************************
;****		Initial I/O Port 
;***********************************************************************
Init_IO:
			clrf	TRISD					; Set PORTD for output port
			clrf	PORTD
   			bsf		TRISA,RA0				; Set RA0 with input mode
			return
;
;***********************************************************************
;****		Initial Timer1 as a 1 Sec Timer 
;***********************************************************************
Init_Timer1:
			movlw	B'10001111'				; 16-bit mode , Prescaler = 1:1 , Async mode
			movwf	T1CON					; Use external 32768Hz crystal , Timer1 On
;
	
			movlw	(.65536-TMR1_VAL)/.256	; Calcute the MSB value for the Timer1
			movwf	TMR1H
			movlw	(.65536-TMR1_VAL)%.256	; Calcute the LSB value for the Timer1
			movwf	TMR1L
;
			bsf		IPR1,TMR1IP				; Set Timer1 for high priority interrupt
			bcf		PIR1,TMR1IF				; Clear Timer1 Interrupt flag
			bsf		PIE1,TMR1IE				; Enable Timer1 Interrupt
;
			return
;
;***********************************************************************
;****		Initial A/D converter 
;***********************************************************************
Init_AD:
			movlw	b'10001110'				; Right Justified, Fosc/32
			movwf	ADCON1					; Set AN0 for the Analog Input
;
  			movlw	b'10000001'				; Fosc/32 , Select AN0 channel
  			movwf	ADCON0					; A/D on
;
			bcf		PIE1,ADIE				; Disable interrupt of A/D

			return
;
;***********************************************************************
;****		Initial USART as 9600,N,8,1 
;***********************************************************************
Init_USART:
			movlw	b'00100000'				; 8-bit data mode , ASYNC 
			movwf	TXSTA					; Low Speed mode, Enable TxD
;
			movlw	b'10010000'				; Enable Serial Port, 8-bit receive
			movwf	RCSTA					; Continuous Receive, Disable Address Detection
;
			movlw	.25						; Set baud rate at 9600 with Low Speed mode
			movwf	SPBRG
;
			bcf		PIR1,TXIF				; Clear TxD interrupt flag
			bcf		PIE1,TXIE				; Disable TxD interrupt
;
			bcf		IPR1,RCIP	  			; Set the RxD for Low priority interrupt
			bcf		PIR1,RCIF				; Clear RxD interrupt flag
			bsf		PIE1,RCIE				; Enable RxD interrupt
;
			return

;
;***********************************************************************
;****		Sample Hold (Charge) time delay routine (50uS) 
;***********************************************************************
C_Hold_Time:
   			movlw	.50
   			movwf	C_Hold_Delay
   			nop
   			decfsz	C_Hold_Delay,F
   			bra		$-4
   			return

;***************************************************************************************
;****		Hi_ISRs : Hi-Priority Interrupt reotine 
;****
;***************************************************************************************
Hi_ISRs

			bcf		PIR1,TMR1IF				; Clear Timer1 Interrupt flag
;							   		
			movlw	(.65536-TMR1_VAL)/.256	; Calcute the MSB value for the Timer1
			movwf	TMR1H
			movlw	(.65536-TMR1_VAL)%.256	; Calcute the LSB value for the Timer1
			movwf	TMR1L
;
			bsf		TxD_Flag,0				; Set 1 Sec time-out flag
;
			retfie	FAST				 	; Return with shadow register
; 
;***************************************************************************************
;****		Low_ISRs : Low-Priority Interrupt reotine 
;****
;***************************************************************************************
Low_ISRs:
			movff	STATUS,STATUS_TEMP		; save STATUS register
			movff	WREG,WREG_TEMP			; save working register
			movff	BSR,BSR_TEMP			; save BSR register
;
			bcf		PIR1,RCIF				; Clear RxD interrupt flag
			movff	RCREG,PORTD				; Display the Rx data to LED port
;
			movlw	a'c'					; Check the received byte is "c" ?
			cpfseq  RCREG
			bra		No_EQU_C
			bcf		Pause_Flag 				; Yes, enable A/D transmitted
			bsf		Send_C_Flag				; Set "send string_7" flag
			clrf	RCREG					; Clear the receive register
			bra		Exit_Low_ISR
No_EQU_C	movlw	a'p'					; Check the received byte is "p" ?
			cpfseq  RCREG
			bra		Exit_Low_ISR			; No, not action
			bsf		Pause_Flag				; Yes, disable the A/D transmitted
			bsf		Send_P_Flag				; Set "send string_6" flag
			clrf	RCREG	
;
Exit_Low_ISR
			movff	BSR_TEMP,BSR			; restore BSR register
			movff	WREG_TEMP,WREG			; restore working register
			movff	STATUS_TEMP,STATUS		; restore STATUS register
			retfie		
; 
;******************************************************************************************
;****		Read a string form Program memory
;****		Send the data to USART until find NULL character
;******************************************************************************************
Put_String
			?????							; Read Table data and save to TABLAT register, Table point + 1
			?????
			?????							; Does Read NULL character ?
			goto	Send_String				; No, send the table data to RS-232
			return							; Yes, return back
;
Send_String	movf	TABLAT,W				; Send the data to USART
			call	Tx_a_Byte				;  
			goto	Put_String				; Read next table dtat
;
;*************************************************************************************
;****		Define display string and control data for the VT-100 terminal 
;*************************************************************************************
;
String_0	db	0x1b,'[','2','J',0x00		; Clear Terminal Screen
String_1	db	"         Microchip Technology Taiwan ; Workshop 400",0x0a,0x0d,0x00
String_2	db  "          Exercise 4 : Know the Table Read / Write ",0x0a,0x0d,0x00
String_3	db	"---------------------------------------------------------------"0x0a,0x0d,0x00
String_4	db  " Press 'p' to pause the A/D and press'c'to send the A/D result",0x0a,0x0d,0x00
String_5	db	" The A/D Results are : ", 0x00
String_6	db	" The A/D is stopped conversion !",0x0a,0x0d,0x00
String_7 	db  " The A/D is running !",0x0a,0x0d,0x00
;
	 	  	END					 
